@@ -23,8 +23,6 @@
  *   http://www.mozilla.org/MPL/                                           *
  ***************************************************************************/
 
-#include <tdebug.h>
-#include <tstring.h>
 #include "asfproperties.h"
 
 using namespace TagLib;
@@ -32,30 +30,30 @@ using namespace TagLib;
 class ASF::Properties::PropertiesPrivate
 {
 public:
-  PropertiesPrivate(): length(0), bitrate(0), sampleRate(0), channels(0), encrypted(false) {}
-  int length;
-  int bitrate;
-  int sampleRate;
-  int channels;
-  bool encrypted;
+  int length { 0 };
+  int bitrate { 0 };
+  int sampleRate { 0 };
+  int channels { 0 };
+  int bitsPerSample { 0 };
+  ASF::Properties::Codec codec { ASF::Properties::Unknown };
+  String codecName;
+  String codecDescription;
+  bool encrypted { false };
 };
 
 ////////////////////////////////////////////////////////////////////////////////
 // public members
 ////////////////////////////////////////////////////////////////////////////////
 
-ASF::Properties::Properties() : AudioProperties(AudioProperties::Average)
+ASF::Properties::Properties() :
+  AudioProperties(AudioProperties::Average),
+  d(std::make_unique<PropertiesPrivate>())
 {
-  d = new PropertiesPrivate;
 }
 
-ASF::Properties::~Properties()
-{
-  if(d)
-    delete d;
-}
+ASF::Properties::~Properties() = default;
 
-int ASF::Properties::length() const
+int ASF::Properties::lengthInMilliseconds() const
 {
   return d->length;
 }
@@ -75,6 +73,26 @@ int ASF::Properties::channels() const
   return d->channels;
 }
 
+int ASF::Properties::bitsPerSample() const
+{
+  return d->bitsPerSample;
+}
+
+ASF::Properties::Codec ASF::Properties::codec() const
+{
+  return d->codec;
+}
+
+String ASF::Properties::codecName() const
+{
+  return d->codecName;
+}
+
+String ASF::Properties::codecDescription() const
+{
+  return d->codecDescription;
+}
+
 bool ASF::Properties::isEncrypted() const
 {
   return d->encrypted;
@@ -84,28 +102,64 @@ bool ASF::Properties::isEncrypted() const
 // private members
 ////////////////////////////////////////////////////////////////////////////////
 
-void ASF::Properties::setLength(int length)
+void ASF::Properties::setLengthInMilliseconds(int value)
 {
-  d->length = length;
+  d->length = value;
 }
 
-void ASF::Properties::setBitrate(int length)
+void ASF::Properties::setBitrate(int value)
 {
-  d->bitrate = length;
+  d->bitrate = value;
 }
 
-void ASF::Properties::setSampleRate(int length)
+void ASF::Properties::setSampleRate(int value)
 {
-  d->sampleRate = length;
+  d->sampleRate = value;
 }
 
-void ASF::Properties::setChannels(int length)
+void ASF::Properties::setChannels(int value)
 {
-  d->channels = length;
+  d->channels = value;
 }
 
-void ASF::Properties::setEncrypted(bool encrypted)
+void ASF::Properties::setBitsPerSample(int value)
 {
-  d->encrypted = encrypted;
+  d->bitsPerSample = value;
 }
 
+void ASF::Properties::setCodec(int value)
+{
+  switch(value)
+  {
+  case 0x0160:
+    d->codec = WMA1;
+    break;
+  case 0x0161:
+    d->codec = WMA2;
+    break;
+  case 0x0162:
+    d->codec = WMA9Pro;
+    break;
+  case 0x0163:
+    d->codec = WMA9Lossless;
+    break;
+  default:
+    d->codec = Unknown;
+    break;
+  }
+}
+
+void ASF::Properties::setCodecName(const String &value)
+{
+  d->codecName = value;
+}
+
+void ASF::Properties::setCodecDescription(const String &value)
+{
+  d->codecDescription = value;
+}
+
+void ASF::Properties::setEncrypted(bool value)
+{
+  d->encrypted = value;
+}
